/*
 *	sample code for TI TLK100 functions  
 *                                       
 *	TI TLK100 Ethernet transceiver.     
 *	                                     
 *  The code was written in C using Microsoft Visual Studio 6.0 IDE
 *  Debugging was done by interfacing Spirent SmartBits-200/2000(SMB-200/2000).
 *
 *  main.c is for debugging and demonstration.
 *  main (main.c) calls the various functions, sets their input variables and  
 *  Handles their output; its purpose is for debugging and demonstration.
 *
 */
	                                     
	                                    





#include < stdio.h > 
#include < string.h >
#include "registers.h"
#include "status.h"
#include "settings.h"
#include "loopback.h"
#include "power_save.h"
#include "prbs.h"
#include "reset.h"
#include "ALCD.h"
#include "Resource files/et1000.h"  
#include "TDR.h"


void main()
{

///////////////////////////////////////////////////////////////////	
//    varaiable declarations	                                 //
//                                                               //
///////////////////////////////////////////////////////////////////

    //general variables
      int error = 0;
	  int phy_id =1; //phy_identification
	
	//link setup variables
	  enum Set_link_behaviors_t Set_link_behavior;
	  enum Set_speeds_t Set_speed;
	  enum Set_duplexes_t Set_duplex;
	  enum Set_MDIs_t Set_mdi;
    
	//loopback variables
	  enum loopback_modes_t loopback_mode;

	//power_save variable
	  enum power_save_modes_t power_save_mode; 

	//prbs variables
      int set_packet_length;
      int set_packet_IPG;
	  int prbs_lock, prbs_sync_loss, prbs_busy;
	  enum prbs_generator_enable_t prbs_generator_enable;
	  enum lock_and_clear_prbs_counters_t lock_and_clear_prbs_counters;
	  enum prbs_count_mode_t prbs_count_mode; 
      prbs_counters prbs_check_counters;

	//ALCD variables
	  int result_alcd;

    //status
      status_phy status_out;

    //TDR variables
      int  reflection[10];
	  int  impedance[10];
	  enum averaging_t averaging;
	  int i;
	  int channel_peak_index;
	  char *channel;


///////////////////////////////////////////////////////////////////	
//    establish link with smartbit (debug)                       //
//                                                               //
///////////////////////////////////////////////////////////////////
  
	error = ETSocketLink("137.167.24.23", 16385);
	if (error < 0)
	{
		printf("Error linking to chassis: %d \n",error);
		error = 0;
	}
 
///////////////////////////////////////////////////////////////////	
//    main function calls	                                     //
//                                                               //
///////////////////////////////////////////////////////////////////


	// reset
	   error = reset_phy(phy_id);
       Sleep(500);


	// link_setup
	   Set_link_behavior = AUTO_NEG;
	   Set_speed = SPD_100;
	   Set_duplex = FD;
	   Set_mdi = MDIX;
       error = link_setup(phy_id,Set_link_behavior,Set_speed,Set_duplex,Set_mdi);
        	if (error < 0)
			{
		         printf("Error link_setup: %d \n",error);
				 error = 0;
			}
       Sleep(1000);
      
    // loopback
       loopback_mode = LOOPBACK_NORMAL;
       error = loopback(phy_id,loopback_mode);
        	if (error < 0)
			{
		         printf("Error loopback: %d \n",error);
				 error = 0;
			}


    // power_save
       power_save_mode = POWER_NORMAL;
	   error = power_save(phy_id,power_save_mode);
        	if (error < 0)
			{
		         printf("Error power_save: %d \n",error);
				 error = 0;
			}


    //prbs_generator
	  set_packet_length = 1500;  //Length of PRBS packets in bytes
      set_packet_IPG = 125;      //Inter-packet gap (in bytes) between PRBS packets
      prbs_count_mode = CONTINUOUS;
      prbs_generator_enable = DISABLE;
	  error = prbs_generator(phy_id, prbs_generator_enable, set_packet_length,set_packet_IPG, prbs_count_mode);
        	if (error < 0)
			{
		         printf("Error prbs_generator: %d \n",error);
				 error = 0;
			}
      Sleep(500);

   //prbs_status
     error = prbs_status(phy_id, &prbs_lock, &prbs_sync_loss, &prbs_busy);
        	if (error < 0)
			{
	 	         printf("Error prbs_status: %d \n",error);	
				 error = 0;
			}
            else
            {
				if (prbs_lock == 1)
				{
                 printf("PRBS checker is locked on received byte stream \n");
				}
				else
				{
                 printf("PRBS checker is not locked \n");
				}
				if (prbs_sync_loss == 1)
				{
                 printf("PRBS checker has lost sync \n");
				}
				else
				{
                 printf("PRBS checker has not lost sync \n");
				}
				if (prbs_busy == 1)
				{
                 printf("Packet generator is in process \n");
				}
				else
				{
                 printf("Packet generator is not in process \n");
				}


            }

   //prbs_checker
  	 lock_and_clear_prbs_counters = LOCK_AND_CLEAR; 
     error = prbs_checker(phy_id, lock_and_clear_prbs_counters, &prbs_check_counters);
        	if (error < 0)
			{
	 	         printf("Error prbs_checker: %d \n",error);	
				 error = 0;
			}
            else
            {
                 printf("byte counter %x  ___ error counter %x \n", prbs_check_counters.byte_counter, prbs_check_counters.error_counter );
            }
     

   //ALCD (Active Link Cable Diagnostic)
     error = ALCD(phy_id, &result_alcd);
        	if (error < 0)
			{
	 	         printf("Error prbs_checker: %d \n",error);
				 error = 0;
			}
            else
            {
				if (result_alcd <0)
				{
					printf("alcd_cable_length: no info \n");
				}
				else
				{
                    printf("alcd_cable_length: %d \n", result_alcd);
				}

            }

      
   //status of the phy
	 Sleep(3000);
       error  = status(phy_id, &status_out);
        	if (error < 0)
			{
		         printf("Error status: %d \n",error);
				 error = 0;
			}
            else
            {
	        switch(status_out.link_status)
			{
			case LINK_UP:
				printf("link is up \n");
			    break;

			case LINK_DOWN:
				printf("link is down \n");
			    break;

			default: //link_down
				printf("link is down \n");
			    break;
			}

            switch(status_out.speed_status)
			{
			case SPD_100:
				printf("speed 100BT \n");
			    break;

			case SPD_10:
				printf("speed 10BT \n");
			    break;

			default: //SPD_10
				printf("SPD_10 \n");
			    break;
			}

            switch(status_out.duplex_status)
			{
			case FD:
				printf("full duplex \n");
			    break;

			case HD:
				printf("half duplex \n");
			    break;

			default: //HD
				printf("half duplex \n");
			    break;
			}

            switch(status_out.pair_swap)
			{
			case MDIX:
				printf("MDIX \n");
			    break;

			case MDI:
				printf("MDI \n");
			    break;

			default: //MDI
				printf("MDI \n");
			    break;
			}
 
            }

   //TDR	 
	 averaging = DISABLE_AVG; //disable or enable averaging		
     error  = TDR( phy_id, reflection, impedance, averaging);

	 printf("TDR \n");
     if (error < 0)
	 {
          printf("TDR error \n");
		  error = 0;
	 }
	 else
	 {

          printf("                  reflection(m)    impedance \n");
          printf("                  --\n");
          for (i=0;i<10;i++)
		  {

	           //select channel to present
	           if (i < 5)
			   {
		            channel = "channel_A";
		            channel_peak_index = i;
			   }
	           else
			   {
		            channel = "channel_B";
		            channel_peak_index = i-5;
			   }

               if (i == 5)
			   {
		            printf("\n"); //space between channels
			   }



               switch(reflection[i])
			   {
	                case REFLECTION_LESS_THAN_3M:
		                 printf("%s peak %d |    <3m             ",channel, channel_peak_index);
	                break;

	                case REFLECTION_TERMINATED:
			             printf("%s peak %d |    terminated         ",channel, channel_peak_index);
                    break;

	                case REFLECTION_NO_INFO:
			             printf("%s peak %d |    no info         ",channel, channel_peak_index);
                    break;
                    //17_10_10
	                case REFLECTION_FAIL:
			             printf("%s peak %d |    fail         ",channel, channel_peak_index);
                    break;
					//17_10_10
					default:
						if ( reflection[i] < -7 || reflection[i] > 250) //undefined number
						{
                         printf("%s peak %d |    no info         ",channel, channel_peak_index);
						}
						else
						{
			             printf("%s peak %d |    %d              ",channel, channel_peak_index ,reflection[i]);
						}
                    break;
			   }


               switch(impedance[i])
			   {
	                case IMPEDANCE_OPEN:
		                 printf("open\n");
	                break;

	                case IMPEDANCE_SHORT_OR_LOW:
		                 printf("short or low\n");
	                break;

	                case IMPEDANCE_BAD:
		                 printf("bad\n");
	                break;

	                case IMPEDANCE_TERMINATED:
		                 printf("terminated\n");
	                break;

	                case IMPEDANCE_NO_INFO:
			             printf("no info\n");
                    break;

	                case IMPEDANCE_NULL:
			             printf("    \n");
                    break;
                    //17_10_10
                    default:
						if ( impedance[i] < 0 ) //undefined number
						{
                         printf("no info\n");
						}
						else
						{
			             printf("%d\n", impedance[i]);
						}  																																						
                    break;

			   }



		  }  


          printf("                  --\n");
	

		 }

  

	ETUnLink();


}

