
/*
 *                                                        
 *  power_save (power_save.c): 
 *	Normal Mode: Normal operation
 *	IEEE Mode: Powering down all digital and analog blocks initiates a write to 
 *  PHY Control Register address 0x0010 bits '9:8' = '01'
 *	Active Sleep: This mode is the same as Passive Sleep mode,
 *  but also sends an NLP every 1.4 seconds (approximately) to wake up a link-partner. 
 *  Initiates a write to PHY Control Register address 0x0010 bits '9:8' = '10'
 *	Passive Sleep: This option powers down all digital and analog blocks.
 *  Automatic power-up is done when the link partner is detected. 
 *  Initiates a write to PHY Control Register address 0x0010 bits '9:8' ='11'
 *
 */                                                        


#include "registers.h"
#include "read_write.h"
#include "status.h"
#include "loopback.h"
#include "power_save.h"


int power_save(int phy_id, int set_power_save)
{

	    int  error=0;
		int  phycr_data=0,bmcr_data=0;

      //set power save
		switch (set_power_save)
        {
         case POWER_NORMAL: //
         phycr_data = phy_read(phy_id, PHYCR); // read PHYCR_DATA holds the value of PHYCR 	   
    		 if (phycr_data < 0)// error
			 {			   
	              return error = phycr_data;
			 }		 
         phycr_data &= 0xFCEF ;   // clear bits [9:8] and clear bit 4		
         error = phy_write(phy_id, PHYCR, phycr_data); //write
	         if (error < 0)
			 {			   
			     return error;
			 }

		 bmcr_data = phy_read(phy_id, BMCR); // read BMCR_DATA holds the value of BMCR 	   
 		     if (bmcr_data < 0)// error
			 {			   
	             return error = bmcr_data;
			 }		 
         bmcr_data &= 0xF7FF ;   // clear bit [11]		
         error = phy_write(phy_id, BMCR, bmcr_data); //write
	         if (error < 0)
			 {			   
			     return error;
			 }
		 break;

		 case POWER_IEEE: // ieee mode
         phycr_data = phy_read(phy_id, PHYCR); // read PHYCR_DATA holds the value of PHYCR 	   
  		     if (phycr_data < 0)// error
			 {			   
	             return error = phycr_data;
			 }		 
         phycr_data &= 0xFCEF ;   // clear bits [9:8] and bit 4		
         phycr_data |= 0x0110 ;   // set bit 8 and set bit 4
         error = phy_write(phy_id, PHYCR, phycr_data); //write
	         if (error < 0)
			 {			   
			     return error;
			 }

		 bmcr_data = phy_read(phy_id, BMCR); // read BMCR_DATA holds the value of BMCR 	   
  		     if (bmcr_data < 0)// error
			 {			   
	             return error = bmcr_data;
			 }		 
         bmcr_data &= 0xF7FF ;   // clear bit [11]		
         bmcr_data |= 0x0800 ;   // set bit 11
         error = phy_write(phy_id, BMCR, bmcr_data); //write
	         if (error < 0)
			 {			   
			     return error;
			 }
		 break;

		 case POWER_ACTIVE_SLEEP: //
         phycr_data = phy_read(phy_id, PHYCR); // read PHYCR_DATA holds the value of PHYCR 	   
  		     if (phycr_data < 0)// error
			 {			   
	             return error = phycr_data;
			 }		 
         phycr_data &= 0xFCEF ;   // clear bits [9:8] and bit 4		
         phycr_data |= 0x0200 ;   // set bit 9 
         error = phy_write(phy_id, PHYCR, phycr_data); //write
	         if (error < 0)
			 {			   
			     return error;
			 }

		 bmcr_data = phy_read(phy_id, BMCR); // read BMCR_DATA holds the value of BMCR 	   
  		     if (bmcr_data < 0)// error
			 {			   
	             return error = bmcr_data;
			 }		 
         bmcr_data &= 0xF7FF ;   // clear bit [11]		
         bmcr_data |= 0x0800 ;   // set bit 11
         error = phy_write(phy_id, BMCR, bmcr_data); //write
	         if (error < 0)
			 {			   
			     return error;
			 }
		 break;

		 case POWER_PASSIVE_SLEEP: //
         phycr_data = phy_read(phy_id, PHYCR); // read PHYCR_DATA holds the value of PHYCR 	   
  		     if (phycr_data < 0)// error
			 {			   
	             return error = phycr_data;
			 }		 
         phycr_data &= 0xFCEF ;   // clear bits [9:8] and bit 4		
         phycr_data |= 0x0300 ;   // set bit 9 and bit 8
         error = phy_write(phy_id, PHYCR, phycr_data); //write
	         if (error < 0)
			 {			   
			     return error;
			 }

		 bmcr_data = phy_read(phy_id, BMCR); // read BMCR_DATA holds the value of BMCR 	   
  		     if (bmcr_data < 0)// error
			 {			   
	             return error = bmcr_data;
			 }		 
         bmcr_data &= 0xF7FF ;   // clear bit [11]		
         bmcr_data |= 0x0800 ;   // set bit 11
         error = phy_write(phy_id, BMCR, bmcr_data); //write
	         if (error < 0)
			 {			   
			     return error;
			 }
		 break;


		 default: // normal power
         phycr_data = phy_read(phy_id, PHYCR); // read PHYCR_DATA holds the value of PHYCR 	   
  		     if (phycr_data < 0)// error
			 {			   
	             return error = phycr_data;
			 }		 
         phycr_data &= 0xFCEF ;   // clear bits [9:8] and clear bit 4		
         error = phy_write(phy_id, PHYCR, phycr_data); //write
	         if (error < 0)
			 {			   
			     return error;
			 }

		 bmcr_data = phy_read(phy_id, BMCR); // read BMCR_DATA holds the value of BMCR 	   
  		     if (bmcr_data < 0)// error
			 {			   
	             return error = bmcr_data;
			 }		 
         bmcr_data &= 0xF7FF ;   // clear bit [11]		
         error = phy_write(phy_id, BMCR, bmcr_data); //write
	         if (error < 0)
			 {			   
			     return error;
			 }
		 break;

		}

		return 0;
}



